#!/usr/bin/env python3
# 25_txt_nach_xml.py
import re
import sys

'''Dies ist ein kleines Beispielprogramm, was verdeutlichen soll, wie man ein Buch aus einer Textdatei in XML umwandeln kann. Allerdings ist es hinsichtlich der Konvertieroptionen extrem beschränkt, da das Eingabeformat u.U. rein auf Hesses Siddhartha beschränkt ist, und deshalb nicht unbedingt generell eingesetzt werden kann. Auch sollten eigentlich die Textteile & Kapitel ebenfalls in Tagklammern eingeschlossen werden, worauf wir hier verzichtet haben, um das Programm nicht zu komplex zu machen.'''

n_teilueberschrift = 0
n_kapitel = 0
n_absatz = 0
n_satz = 0

def satz_erzeugen(match_objekt):
    satz = match_objekt.group(1)
    global n_satz
    n_satz += 1
    return f'<satz n="{n_satz}">\n{satz}\n</satz>\n'

# Eingabedatei einlesen
try:
    with open('./hesse_siddhartha.txt', mode='r', encoding='utf-8') as ein:
        absaetze = re.split(r'\n{2,}', ein.read())
except OSError as fehler:
    sys.exit(str(fehler))

try:
    with open('./hesse_siddhartha.xml', mode='w', encoding='utf-8') as aus:
        # 'header' schreiben
        aus.write('<?xml version="1.0"?>\n')
        aus.write('<text id="siddhartha" corpus="hesse" lang="de">\n')
        for absatz in absaetze:
            # Zeilen in Absatz zusammenfügen
            absatz = re.sub(r'\n', ' ', absatz)
            absatz = re.sub(r' +', ' ', absatz)
            if re.search(r'^[A-ZÄÖÜ]{2,}\b', absatz):
                if re.search(r'\bTEIL\b', absatz):
                    n_teilueberschrift += 1
                    aus.write(f'<teil n="{n_teilueberschrift}">\n{absatz}\n</teil>\n')
                else:
                    n_kapitel += 1
                    aus.write(f'<kapitel n="{n_kapitel}">\n{absatz}\n</kapitel>\n')
                continue
            else:
                n_absatz += 1
                absatz=re.sub(r'([\w, ":;-]+?[.?!]"?)(?:\s+|$)', satz_erzeugen, absatz)
            aus.write(f'<absatz n="{n_absatz}">\n{absatz}</absatz>\n')
        # 'footer' schreiben
        aus.write('</text>')
        
except OSError as fehler:
    sys.exit(str(fehler))