#!/usr/bin/env python3
# GUI_syn_inversion.py

import sys
import re
from PyQt5.QtWidgets import (QMainWindow,QApplication,
                             QHBoxLayout,QVBoxLayout,QLabel,QLineEdit,
                             QPushButton,QFrame,QMessageBox)
from PyQt5.QtGui import QFont

class Inverter(QMainWindow):
    """GUI zur Illustration syntaktischer Inversion"""

    def __init__(self):
        # Konstruktor der Elternklasse aufrufen
        super().__init__()
        self.setFont(QFont("Courier", 12))
        self.initUI()
        
    def initUI(self):
        # Behälter für Layouts anlegen
        behaelter = QFrame()
        # Behälter als zentrales Steuerelement setzen
        self.setCentralWidget(behaelter)
        # Haupt- und Unterlayouts definieren
        haupt_layout = QVBoxLayout()
        dekl_layout = QHBoxLayout()
        interr_layout = QHBoxLayout()
        konv_layout = QHBoxLayout()
        # Unter-Layouts zum Hauptlayout hinzufügen
        haupt_layout.addLayout(dekl_layout)
        haupt_layout.addLayout(interr_layout)
        haupt_layout.addLayout(konv_layout)
        # Behälter Hauptlayout zuweisen
        behaelter.setLayout(haupt_layout)
       
        #Steuerelemente definieren
        self.dekl_eingabe = QLineEdit('Dies ist ein Deklarativsatz.')
        self.interr_ausgabe = QLineEdit()
        konverter = QPushButton('Konvertieren')
        # Signal mit Slot verknüpfen
        konverter.clicked.connect(self.inversion)
        
        # Steuerelemente in Layouts einfügen
        dekl_layout.addWidget(QLabel('Deklarativsatz:\t'))
        dekl_layout.addWidget(self.dekl_eingabe)
        konv_layout.addWidget(konverter)
        interr_layout.addWidget(QLabel('Interrogativsatz: '))
        interr_layout.addWidget(self.interr_ausgabe)
    

    def inversion(self):
        teile = re.search(r'^(\w+\b) (\b\w+\b)(.+?)\.$',self.dekl_eingabe.text())
        if self.dekl_eingabe.text() and teile:
            self.interr_ausgabe.setText(
                f'{teile.group(2).capitalize()} {teile.group(1).lower()}{teile.group(3)}?')
        else:
            QMessageBox(
                text='Bitte einen kompletten einfachen'
                'Deklarativsatz eingeben!!',    
                windowTitle='Eingabefehler',
                icon=QMessageBox.Critical).exec_()

if __name__ == '__main__':
    # Instantiierung des Programms
    app = QApplication(sys.argv)
    # Anlegen des Fensterobjekts
    inverter = Inverter()
    inverter.setWindowTitle('Einfache syntaktische Inversion')
    inverter.setGeometry(15,35,600,100)
    # Anzeigen des Fensters
    inverter.show()
    # Programmaufruf
    sys.exit(app.exec_())