#!/usr/bin/env python3
# frequenzen.py
# Sun Oct-20-2019 07:19:53

import re
import os.path

class Frequenzliste():
    
    def __init__(self,
                 eingabe_datei=None,
                 ausgabe_datei=None,
                 sortierung='n-1'):
        self.woerter = {}
        self.sortiert = []
        if not eingabe_datei:
            raise NameError('Keine Eingabedatei angegeben! Unmöglich, Frequenzliste anzulegen...')
        else:
            pfad, dateiname = os.path.split(eingabe_datei)
            self.eingabe_datei = eingabe_datei
        if not ausgabe_datei:
            self.ausgabe_datei = os.path.join(pfad, 'frequenzliste_' + eingabe_datei)
        else:
            self.ausgabe_datei = ausgabe_datei
        self.sortierung = sortierung
            
    def liste_erzeugen(self):
        self.woerter.clear()
        self.laengstes_wort = 0
        self.max_laenge_zahl = 0
        try:
            with open('./' + self.eingabe_datei, 'r', encoding='utf-8') as datei:
                for zeile in datei:
                    zeile = re.sub(r'[»«.,;!?:-]','', zeile)
                    zeile = re.sub(r'\s{2,}', ' ', zeile)
                    zeile = zeile.strip()
                    if not zeile:
                        continue
                    for wort in re.split(r'\s', zeile):
                        self.woerter[wort] = self.woerter.setdefault(wort, 0) + 1
                        if len(wort) > self.laengstes_wort:
                            self.laengstes_wort = len(wort)
                        if len(str(self.woerter[wort])) > self.max_laenge_zahl:
                            self.max_laenge_zahl = len(str(self.woerter[wort]))
        except OSError as fehler:
            raise OSError(fehler)
        
    def liste_sortieren(self):
        if self.sortierung=='a-z':
            self.sortiert = sorted(self.woerter.keys(), key=str.casefold)
        elif self.sortierung=='z-a':
            self.sortiert = sorted(self.woerter.keys(), key=str.casefold, reverse=True)
        elif self.sortierung=='n-1':
            self.sortiert = sorted(self.woerter.keys(), key=lambda wort: (-self.woerter[wort],wort.casefold()))
        elif self.sortierung=='w_laenge':
            self.sortiert = sorted(self.woerter.keys(), key=lambda wort: (-len(wort),-self.woerter[wort],wort))
        elif self.sortierung=='rueck':
            self.sortiert = sorted(self.woerter.keys(), key=lambda wort: (wort[::-1],len(wort)))
        else:
            self.sortiert = sorted(self.woerter.keys(), key=lambda wort: (self.woerter[wort],wort.casefold()))
    
    def liste_ausgeben(self):
        try:
            with open(self.ausgabe_datei, 'w', encoding='utf-8') as datei:
                for wort in self.sortiert:
                    datei.write(
                        f'{wort:{self.laengstes_wort}}\t'
                        f'{self.woerter[wort]:>{self.max_laenge_zahl}d}\n')                    
        except OSError as fehler:
            raise OSError(fehler)

if __name__ == '__main__':
    try:
        f_liste = Frequenzliste(eingabe_datei='./kafka_verwandlung.txt',ausgabe_datei='./test_frequenzliste.txt',sortierung='n-1')
        f_liste.liste_erzeugen()
        f_liste.liste_sortieren()
        f_liste.liste_ausgeben()
    except OSError as f1:
        print('Falsche Ein- oder Ausgabedatei angegeben!', str(f1).split(' ')[-1])
    except NameError as f2:
        print(str(f2))