#!/usr/bin/env python3
# wort.py
import re
import sys

class Verb:
    """Klasse zur Modellierung von Verben
    
    infinitiv -- Infinitivform: nicht optional
    person -- Person: Standardwert '1'
    form_grad -- Formalitätsgrad: Standardwert 'informell'
    numerus -- Numerus: Standardwert 'singular'
    tempus -- Tempus: Standardwert 'praesens'
    modus -- Modus: Standardwert 'indikativ'
    typ -- Typ: Standardwert 'r' für regulär
    """

    def __init__(self, 
                 infinitiv=None, 
                 person='1', 
                 form_grad='informell', 
                 numerus='singular', 
                 tempus='praesens', 
                 modus='indikativ', 
                 typ='r'):
        """Konstruktor"""
        if not infinitiv:
            raise NameError('Keine Infinitivform angegeben!')
        self.infinitiv = infinitiv
        self.person = person
        self.form_grad = form_grad
        self.numerus = numerus
        self.tempus = tempus
        self.typ = typ
        if not re.search(r'[lr]',self.infinitiv[-2:-1]):
            self.stamm = self.infinitiv[:-2]
        else:
            # Liquid vor Infinitiv-Endung
            self.stamm = self.infinitiv[:-1]
    
    def partizip2(self):
        if self.typ=='r':
            # finaler Frikativ, Langvokal, Liquid, 
            # nicht-alveolarer Plosiv, oder Doppelkonsonant
            # am Stammende
            if re.search(r'(?:c?h|(?:ie|(?:[aoäöü]h|r)n?)|[pbfkglmsz])$', 
                         self.stamm)\
               or re.search(r'[nms]{2}$', self.stamm):
                return 'ge' + self.stamm + 't'
            elif re.search(r'(?:[dtn])$', self.stamm):
                # finaler alveolarer plosiv
                return 'ge' + self.stamm + 'et'        
            else:
                return 'ge' + self.stamm + 'en'
        else:
            return 'Irreguläres Verb. Noch nicht implementiert.'        
    
    def partizip1(self):
        return self.infinitiv + 'd'
    
    def praesens(self):
        if self.typ=='r':
            if self.numerus=='singular':
                if self.person=='1':
                    return self.stamm + 'e'
                elif self.person=='2':
                    if self.form_grad=='informell':
                        if re.search(r'[mndt]',
                                     self.infinitiv[-3:-2]) \
                           and not re.search(r'[nm]{2}$', self.stamm):
                            return self.stamm + 'est'
                        else:
                            if re.search(r's$', self.stamm):
                                return self.stamm + 't'
                            else:
                                return self.stamm + 'st'
                    else:
                        if re.search(r'[rl]$', self.stamm):
                            return self.stamm + 'n'
                        else:
                            return self.stamm + 'en'
                else:
                    if re.search(r'[mndt]',
                                 self.infinitiv[-3:-2]) \
                       and not re.search(r'[nms]{2}$', self.stamm):
                        return self.stamm + 'et'
                    else:
                        return self.stamm + 't'
            if self.numerus=='plural':
                if self.person=='2' and \
                   self.form_grad=='informell':
                    if re.search(
                        r'(?:c?h|(?:ie|(?:[aoäöü]h|r)n))$'
                        r'|[pbmfkglnsz]$', self.stamm):
                        return self.stamm + 't'
                    else:
                        return self.stamm + 'et'
                else:
                    if re.search(r'[rl]$', self.stamm):
                            return self.stamm + 'n'
                    else:
                            return self.stamm + 'en'
        else:
            return 'Irreguläres Verb. Noch nicht implementiert.'
        
if __name__ == '__main__':
    for inf in ['fassen', 'duschen', 'lächeln', 'füttern', 
                'flüstern', 'reden', 'trennen', 'prüfen']:
        verb = Verb(infinitiv=inf,
                    person='2',
                    form_grad='formell',
                    numerus='singular')
        print(f'Partizip 1: {verb.partizip1()}, '
              f'Partizip 2: {verb.partizip2()}')
        if verb.person == '1':
            if verb.numerus == 'singular':
                pronomen = 'ich'
            else:
                pronomen = 'wir'
        elif verb.person == '3':
            if verb.numerus == 'singular':
                pronomen = 'er/sie/es'
            else:
                pronomen = 'sie'
        else:
            if verb.numerus == 'singular' and \
               verb.form_grad == 'informell':
                pronomen = 'Du'
            else:
                pronomen = 'Sie'
        print(f'Stamm: {verb.stamm}; Präsens Indikativ für'
              f'{verb.person}. Person {verb.numerus.capitalize()}, '
              f'Formalitätsgrad {verb.form_grad}: {pronomen} '
              f'{verb.praesens()}')